@@ -43,7 +43,7 @@ class ComputeRBF():
     NORMVALUE: matrix with distances of all candidate points to already
     sampled points
     '''
-    def __init__(self,phifunction='linear',polynomial='linear',**kwargs):
+    def __init__(self,phifunction='cubic',polynomial='linear',**kwargs):
         self.phifunction = phifunction
         self.polynomial  = polynomial
         
@@ -82,6 +82,6 @@ class ComputeRBF():
             RBFvalue = numpy.concatenate((c1,c2),axis=1)
             return RBFvalue, numpy.asmatrix(Normvalue)
 
-    def predict(self,X):
+    def predict(self,X,return_std=False):
         return numpy.array(self.fit(X)[0])
         
