@@ -24,7 +24,7 @@ from ..dnnmodels import training
 from ..datasets import get_data
 from ..hyperparams import set_hyperparams
 
-def plot_convergence(loss,stdev,isurr=None,ylim=[]):
+def plot_convergence(loss,stdev,isurr=None,ylim=[],xlim=[]):
     plt.style.use('seaborn')
     plt.figure(figsize=(6,4),dpi=100)
     plt.plot(loss,color='black',lw=0.5,zorder=1,
@@ -36,7 +36,10 @@ def plot_convergence(loss,stdev,isurr=None,ylim=[]):
         plt.axvline(isurr,color='black',lw=1,ls='dashed')
     if len(ylim)>0:
         plt.ylim(ylim)
-    plt.xlim(0,len(loss))
+    if len(xlim)>0:
+        plt.xlim(xlim)
+    else:
+        plt.xlim(0,len(loss))
     plt.gca().yaxis.set_major_formatter(FormatStrFormatter('%.2f'))
     plt.xlabel('Index of function evaluations')
     plt.ylabel('Loss')
@@ -80,6 +83,74 @@ def hpo_solutions(results):
     fig.write_html('solutions.html')
     fig.show()
 
+def model_visualization_1d(X,Y,path,evaluation,surrogate,imax=10,sigma=False,ylim=[-0.5,2.5]):
+    scaler = round(1/(X[1]-X[0]))
+    os.makedirs('%s/plots' % (path),exist_ok=True)
+    plt.style.use('seaborn')
+    for i,hp1s in enumerate(surrogate['hp1']):
+        if i==imax:
+            break
+        model = pickle.load(open('%s/models/surrogate_%i/model.pkl' % (path,hp1s),'rb'))
+        if sigma:
+            Y_pred, Y_sigma = model.predict(X.reshape(-1, 1)*scaler,return_std=True)
+        else:
+            Y_pred = model.predict(X.reshape(-1, 1)*scaler,return_std=False)
+        Y_pred = Y_pred.squeeze()
+        fig,ax = plt.subplots(1,3,figsize=[12,4],dpi=200,sharex=True,sharey=True)
+        fig.suptitle('Surrogate Modeling - Iteration %i / %i - Hyperparameters (%.2f)' % (i+1,imax,hp1s/scaler))
+        ax[0].set_ylim(ylim)
+        ax[0].plot(X,Y,'black')
+        ax[1].plot(X,Y,'black',ls='dashed')
+        ax[1].plot(X,Y_pred,'black')
+        if sigma:
+            ax[1].fill_between(X,Y_pred-Y_sigma/2,Y_pred+Y_sigma/2,alpha=0.2,lw=0,color='blue',step='post')
+        ax[2].plot(X,Y-Y_pred,'black')
+        for k in range(2):
+            ax[k].set_xlabel('X')
+            for hp,loss in zip(evaluation['hp1'],evaluation['loss']):
+                ax[k].plot([hp/scaler], [loss], 'bx', mew=3)
+            for n,(hp,loss) in enumerate(zip(surrogate['hp1'],surrogate['loss'])):
+                if n>i: break
+                ax[k].plot([hp/scaler], [loss], 'rx', mew=3)
+        ax[0].set_ylabel('Y')
+        plt.tight_layout()
+        plt.savefig('%s/plots/iter_%02i' % (path,i+1))
+        plt.close()
+    
+def model_visualization_2d(X,Y,path,evaluation,surrogate,imax=10,sigma=False):
+    scaler = round(1/(X[1]-X[0]))
+    X1, Y1 = numpy.meshgrid(X, Y)
+    R = numpy.sqrt(X1**2 + Y1**2)
+    Z = 0.4 - (1. / numpy.sqrt(2 * numpy.pi)) * numpy.exp(-.5*R**2)
+    Z_scaler = round(Z.max(),1)
+    os.makedirs('%s/plots' % (path),exist_ok=True)
+    plt.style.use('seaborn')
+    for i,(hp1s,hp2s) in enumerate(zip(surrogate['hp1'],surrogate['hp2'])):
+        if i==imax:
+            break
+        model = pickle.load(open('%s/models/surrogate_%i_%i/model.pkl' % (path,hp1s,hp2s),'rb'))
+        comb = numpy.array([(x,y) for y in Y for x in X]).reshape(-1, 2)*scaler
+        Y_pred = model.predict(comb).reshape(len(X),len(Y))
+        Y_pred[Y_pred<0], Y_pred[Y_pred>0.4] = 0.0, Z_scaler
+        fig,ax = plt.subplots(1,3,figsize=[12,4],dpi=200,sharex=True,sharey=True)
+        fig.suptitle('Surrogate Modeling - Iteration %i / %i - Hyperparameters (%.2f, %.2f)' % (i+1,imax,hp1s/scaler,hp2s/scaler))
+        im0 = ax[0].contourf(X1,Y1,Z,levels=numpy.linspace(0,Z_scaler,scaler),cmap='viridis')
+        im1 = ax[1].contourf(X1,Y1,Y_pred,levels=numpy.linspace(0,Z_scaler,scaler),cmap='viridis')
+        im2 = ax[2].contourf(X1,Y1,Z-Y_pred,levels=numpy.linspace(-Z_scaler,Z_scaler,scaler),cmap='bwr')
+        for k in range(3):
+            ax[k].set_xlabel('X')
+            for hp1,hp2 in zip(evaluation['hp1'],evaluation['hp2']):
+                ax[k].plot(hp1/scaler, hp2/scaler, 'kx', mew=3)
+            for n,(hp1,hp2) in enumerate(zip(surrogate['hp1'],surrogate['hp2'])):
+                if n>i: break
+                ax[k].plot(hp1/scaler, hp2/scaler, 'rx', mew=3)
+            cbar = plt.colorbar(eval('im%i' % k), ax=ax[k],ticks=[0,Z_scaler/2,Z_scaler] if k<2 else [-Z_scaler,0,Z_scaler],
+                                shrink=0.8, orientation='horizontal', location='top')
+        ax[0].set_ylabel('Y')
+        plt.tight_layout()
+        plt.savefig('%s/plots/iter_%02i' % (path,i+1))
+        plt.close()
+    
 # class PlotResults():
     
 #     def __init__(self, data, y_predicted, output_name):
